from rdflib import Graph, URIRef, BNode
import rdflib.extras.infixowl as rdfin
import os
import pandas as pd
import glob

class Parse():
    def __init__(self, path):
        try:
            self.graph = Graph().parse(r"..\benchmarks\{}\onto.rdf".format(path))
        except:
            self.graph = Graph().parse(r"..\benchmarks\{}\test.rdf".format(path))
        self.path = path
        self.par = Graph().parse(r"..\benchmarks\{}\refalign.rdf".format(path))

    # five-tuple list, get class，label，comment，father, child
    def getClasses(self):
        classes = [[] for i in range(len(self.graph))]
        sub = URIRef("http://www.w3.org/2000/01/rdf-schema#subClassOf")
        for index, i in enumerate(rdfin.AllClasses(self.graph)):
            # print(str(i.identifier))
            # type <class 'rdflib.extras.infixowl.Class'>
            # 303和302
            id = i.identifier
            if "rdf#" in str(id) or "ontology#" in str(id) or "owl#" in str(id):
                classes[index].append(str(id)[str(id).find('#') + 1:])
                classes[index].append(str(self.graph.label(id)))
                classes[index].append(str(self.graph.comment(id)))
                parents = ""
                for j in list(self.graph.objects(id, sub)):
                    if "#" in str(j):
                        parents += str(j)[str(j).find("#") + 1:] + " "
                parents = parents.strip()
                classes[index].append(parents)
                childen = ""
                for j in list(self.graph.subjects(sub, id)):
                    if "#" in str(j):
                        childen += str(j)[str(j).find("#") + 1:] + " "
                childen = childen.strip()
                classes[index].append(childen)
                classes[index].append("class")
        return [i for i in classes if i]

    def getParsealign(self):
        # 谓语
        align1 = URIRef("http://knowledgeweb.semanticweb.org/heterogeneity/alignmententity1")
        align2 = URIRef("http://knowledgeweb.semanticweb.org/heterogeneity/alignmententity2")
        median = URIRef("http://knowledgeweb.semanticweb.org/heterogeneity/alignmentmeasure")
        bnodes = []
        for i in self.par.subjects(median):
            bnodes.append(i)
        mix = [[] for i in range(len(self.par))]
        for index, i in enumerate(bnodes):
            a = list(self.par.objects(i, align1))[0]
            b = list(self.par.objects(i, align2))[0]
            onto1 = str(a)[str(a).find("#") + 1:]
            onto2 = str(b)[str(b).find("#") + 1:]
            mix[index].append(onto1)
            mix[index].append(onto2)

        return [i for i in mix if i]

    def getProperty(self):
        ran = URIRef("http://www.w3.org/2000/01/rdf-schema#range")
        dom = URIRef("http://www.w3.org/2000/01/rdf-schema#domain")
        unionof = URIRef("http://www.w3.org/2002/07/owl#unionOf")
        first = URIRef("http://www.w3.org/1999/02/22-rdf-syntax-ns#first")
        rest = URIRef("http://www.w3.org/1999/02/22-rdf-syntax-ns#rest")
        properties = [[] for i in range(len(self.graph))]
        for index, i in enumerate(rdfin.AllProperties(self.graph)):
            id = i.identifier
            str_id = str(id)
            # if str_id[str_id.find('#') + 1:] == "publisher":
            #     print(123)
            if "rdf#" in str_id or "ontology#" in str_id or "owl#" in str_id:
                properties[index].append(str_id[str_id.find('#') + 1:])
                try:
                    properties[index].append(str(list(i.label)[0]))
                except:
                    properties[index].append('null')
                try:
                    properties[index].append(str(list(i.comment)[0]))
                except:
                    properties[index].append('null')

                domains = list(i.domain)
                if domains:
                    test = str(domains[0].identifier)
                    if "Some Class" in str(domains[0]):
                        test2 = str(domains[0]).split("'")
                        try:
                            properties[index].append(test2[1] + "," + test2[3] + "," + test2[5])
                        except:
                            try:
                                properties[index].append(test2[1] + "," + test2[3])
                            except:
                                properties[index].append(test2[1])
                    else:
                        properties[index].append(test[test.find('#') + 1:])
                else:
                    properties[index].append('null')
                try:
                    r = list(self.graph.objects(id, ran))[0]
                    properties[index].append(str(r)[str(r).find('#') + 1:])
                except:
                    properties[index].append('null')
                properties[index].append("property")
        properties = [i for i in properties if i]
        properties_df = pd.DataFrame(properties, columns=['property_id', 'property_label', 'property_comment', 'property_domain', 'property_range', 'type'])
        properties_df.to_csv(
            r"..\benchmarks\{}\properties.csv".format(self.path),
            index=False)
        return [i for i in properties if i]

    def getDataproperties(self):
        typ = URIRef("http://www.w3.org/1999/02/22-rdf-syntax-ns#type")
        dataP = URIRef("http://www.w3.org/2002/07/owl#DatatypeProperty")
        allP = self.getProperty()
        dataProperties = [[] for i in range(len(allP))]
        for i in list(self.graph.subjects(typ, dataP)):
            for j in allP:
                stri = str(i)
                if "rdf#" in stri or "ontology#" in stri or "owl#" in stri:
                    if stri[stri.find("#") + 1:] == j[0]:
                        dataProperties.append(j)
                        # dataProperties[-1][5] = "dataproperty"
        dataProperties = [i for i in dataProperties if i]
        return dataProperties

    def getObjectProperties(self):
        typ = URIRef("http://www.w3.org/1999/02/22-rdf-syntax-ns#type")
        objectP = URIRef("http://www.w3.org/2002/07/owl#ObjectProperty")
        allP = self.getProperty()
        ObjectProperty = [[] for i in range(len(allP))]
        for i in list(self.graph.subjects(typ, objectP)):
            for j in allP:
                stri = str(i)
                if "rdf#" in stri or "ontology#" in stri or "owl#" in stri:
                    if stri[stri.find("#") + 1:] == j[0]:
                        ObjectProperty.append(j)
                        # ObjectProperty[-1][5] = "objectproperty"

        ObjectProperty = [i for i in ObjectProperty if i]
        return ObjectProperty

def triple(g):
    for i, j, k in g:
        # print(i,j,k)
        print(type(i), i)
        print(type(j), j)
        print(type(k), k)
        print("~~~")

def parse2file():
    for i in os.walk("benchmarks"):
        for j in i[1]:
            onto = Parse(j)
            print(j)
            # 创建目录
            try:
                os.mkdir(r"parseDir/{}".format(j))
            except:
                pass
            try:
                pd.DataFrame(onto.getClasses()).to_csv(r"Files/parseDir/{}/classes.csv".format(j), encoding='utf-8',
                                                       index=0)
                pd.DataFrame(onto.getDataproperties()).to_csv(r"Files/parseDir/{}/Dataproperties.csv".format(j),
                                                              encoding='utf-8', index=0)
                pd.DataFrame(onto.getObjectProperties()).to_csv(r"Files/parseDir/{}/ObjectProperties.csv".format(j),
                                                                encoding='utf-8',
                                                                index=0)
            except:
                print("mistake" + j)
            try:
                pd.DataFrame(onto.getParsealign()).to_csv(r"Files/parseDir/{}/refalign.csv".format(j), encoding='utf-8',
                                                          index=0)
            except:
                print("mistake" + j)
            break
        break


if __name__ == "__main__":
    onto1 = Parse(304)
    print(onto1.getParsealign())
    print(onto1.getClasses())
    # print((onto1.getProperty()))

