from owlready2 import *
import rdflib
import pandas as pd
from rdflib import Graph, URIRef, BNode, Literal
import glob


class OntologyParsing:
    def __init__(self, path):
        try:
            self.onto = get_ontology(
                r"..\benchmarks\{}\onto.rdf".format(path)).load()
            self.path = path
        except:
            self.onto = get_ontology(
                r"..\benchmarks\{}\test.rdf".format(path)).load()
            self.path = path

    def get_classes(self):
        onto_classes_df = pd.DataFrame(
            columns=['class_id', 'class_label', 'class_comment', 'superclass_id', 'superclass_label',
                     'number_of_superclasses', 'number_of_subclasses', 'degrees'])
        onto_classes = list(self.onto.classes())
        for onto_class in onto_classes:
            if "#" in str(onto_class):
                father = onto_class.is_a
                class_label = onto_class.label
                if class_label:
                    class_label = class_label[0]
                else:
                    class_label = ""
                class_comment = onto_class.comment
                if class_comment:
                    class_comment = class_comment[0]
                else:
                    class_comment = ""
                children = list(onto_class.subclasses())
                number_sup = len(father)
                number_sub = len(children)
                degrees = number_sup + number_sub
                father = father[0]
                father_label = father.label
                # father = str(father)[onto_name_len:]
                if father_label:
                    father_label = father_label[0]
                else:
                    father_label = ""
                if "#" in str(father):
                    father_str = str(father)[str(father).find('#') + 2:]
                else:
                    father_str = str(father)
                onto_classes_df = onto_classes_df.append({'class_id': str(onto_class)[str(onto_class).find('#') + 2:],
                                                          'class_label': class_label,
                                                          'class_comment': class_comment,
                                                          'superclass_id': father_str,
                                                          'superclass_label': father_label,
                                                          'number_of_superclasses': number_sup,
                                                          'number_of_subclasses': number_sub,
                                                          'degrees': degrees}, ignore_index=True)
        onto_classes_df = onto_classes_df.sort_values(by=["degrees"], ascending=[False], ignore_index=True)
        onto_classes_df.to_csv(
            r"..\benchmarks\{}\classes.csv".format(self.path), index=False)
        return onto_classes_df

    def get_properties(self):
        onto_properties_df = pd.DataFrame(
            columns=['property_id', 'property_label', 'property_comment', 'property_domain', 'property_range', 'type'])
        onto_properties = list(self.onto.properties())
        for onto_property in onto_properties:
            if "#" in str(onto_property):
                try:
                    property_label = onto_property.label
                    if property_label:
                        property_label = property_label[0]
                    else:
                        property_label = ""
                except AttributeError:
                    property_label = ""

                try:
                    property_comment = onto_property.comment
                    if property_comment:
                        property_comment = property_comment[0]
                    else:
                        property_comment = ""
                except AttributeError:
                    property_comment = ""

                property_domain = onto_property.domain
                if property_domain:
                    property_domain = property_domain[0]
                else:
                    property_domain = ""

                property_range = onto_property.range
                if property_range:
                    property_range = property_range[0]
                else:
                    property_range = ""

                onto_properties_df = onto_properties_df.append(
                    {'property_id': str(onto_property)[str(onto_property).find('#') + 2:],
                     'property_label': property_label,
                     'property_comment': property_comment,
                     'property_domain': property_domain,
                     'property_range': property_range,
                     'type': "property"}, ignore_index=True)
        onto_properties_df.to_csv(
            r"..\benchmarks\{}\properties.csv".format(self.path),
            index=False)
        print(onto_properties_df)
        return onto_properties_df

    def get_object_properties(self):
        onto_object_properties_df = pd.DataFrame(
            columns=['property_id', 'property_label', 'property_comment', 'property_domain', 'property_range'])
        onto_properties = list(self.onto.object_properties())
        for onto_property in onto_properties:
            if "#" in str(onto_property):
                property_label = onto_property.label
                if property_label:
                    property_label = property_label[0]
                else:
                    property_label = ""

                property_comment = onto_property.comment
                if property_comment:
                    property_comment = property_comment[0]
                else:
                    property_comment = ""

                property_domain = onto_property.domain
                if property_domain:
                    property_domain = property_domain[0]
                else:
                    property_domain = ""

                property_range = onto_property.range
                if property_range:
                    property_range = property_range[0]
                else:
                    property_range = ""

                onto_object_properties_df = onto_object_properties_df.append(
                    {'property_id': str(onto_property)[str(onto_property).find('#') + 2:],
                     'property_label': property_label,
                     'property_comment': property_comment,
                     'property_domain': property_domain,
                     'property_range': property_range}, ignore_index=True)
        onto_object_properties_df.to_csv(
            r"..\benchmarks\{}\object_properties.csv".format(self.path),
            index=False)
        return onto_object_properties_df

    def get_data_properties(self):
        onto_data_properties_df = pd.DataFrame(
            columns=['property_id', 'property_label', 'property_comment', 'property_domain', 'property_range'])
        onto_properties = list(self.onto.data_properties())
        for onto_property in onto_properties:
            if "#" in str(onto_property):
                property_label = onto_property.label
                if property_label:
                    property_label = property_label[0]
                else:
                    property_label = ""

                property_comment = onto_property.comment
                if property_comment:
                    property_comment = property_comment[0]
                else:
                    property_comment = ""

                property_domain = onto_property.domain
                if property_domain:
                    property_domain = property_domain[0]
                else:
                    property_domain = ""

                property_range = onto_property.range
                if property_range:
                    property_range = property_range[0]
                else:
                    property_range = ""

                onto_data_properties_df = onto_data_properties_df.append(
                    {'property_id': str(onto_property)[str(onto_property).find('#') + 2:],
                     'property_label': property_label,
                     'property_comment': property_comment,
                     'property_domain': property_domain,
                     'property_range': property_range}, ignore_index=True)
        onto_data_properties_df.to_csv(
            r"..\benchmarks\{}\data_properties.csv".format(self.path),
            index=False)
        return onto_data_properties_df


class RefAlignParsing:
    def __init__(self, path):
        self.graph = Graph().parse(
            r"..\benchmarks\{}\refalign.rdf".format(path))
        self.path = str(path)

    def get_ref_align(self):
        align1 = URIRef("http://knowledgeweb.semanticweb.org/heterogeneity/alignmententity1")
        align2 = URIRef("http://knowledgeweb.semanticweb.org/heterogeneity/alignmententity2")
        equivalent = Literal("=")
        # get bnodes using equivalent
        bnodes = []
        for i in self.graph.subjects(object=equivalent):
            bnodes.append(i)
        alignments = [[] for i in range(len(bnodes))]
        for index, i in enumerate(bnodes):
            a = list(self.graph.objects(i, align1))[0]
            b = list(self.graph.objects(i, align2))[0]
            onto1 = str(a)[str(a).find("#") + 1:]
            onto2 = str(b)[str(b).find("#") + 1:]
            alignments[index].append(onto1)
            alignments[index].append(onto2)

        alignments = [i for i in alignments if i]
        alignments = sorted(alignments)
        alignment_df = pd.DataFrame(alignments, columns=['onto1', 'onto2'])
        alignment_df.to_csv(
            r"..\benchmarks\{}\RA.csv".format(self.path),
            index=False)
        print(alignment_df)
        return alignment_df

if __name__ == "__main__":
    onto = OntologyParsing(210)
    # onto.get_classes()
    onto.get_properties()