# coding=utf-8
import Levenshtein
import wordninja
from nltk.corpus import wordnet as wn
import pandas as pd
import numpy as np
from nltk.util import ngrams as nltkngram
# from ontology import *

class similirites():

    def __init__(self, s1, s2):
        self.s1 = s1
        self.s2 = s2

    def Levensh(self):
        if self.s1 == "null" or self.s2 == "null":
            return 0
        if not self.s1 or not self.s2:
            return 0
        else:
            minimum = min(len(self.s1), len(self.s2))
            d = Levenshtein.distance(self.s1, self.s2)
            levensh = max(0, (minimum - d) / minimum)
            # print(minimum, d)
            return levensh

    def Jaro(self):
        if self.s1 == "null" or self.s2 == "null":
            return 0
        elif not self.s1 or not self.s2:
            return 0
        else:
            return Levenshtein.jaro(self.s1, self.s2)

    def wordnet(self):
        if self.s1 == "null" or self.s2 == "null":
            return 0
        elif self.s1 == self.s2:
            return 1
        list1 = wn.synsets(self.s1)
        list2 = wn.synsets(self.s2)
        simi = []
        if not list1:
            return 0
        if not list2:
            return 0
        else:
            for i in list1:
                for j in list2:
                    if i.path_similarity(j):
                        simi.append(i.path_similarity(j))
            if not simi:
                return 0
            else:
                return max(simi)

    def equal(self):
        if self.s1 == self.s2:
            return 1
        else:
            return 0

    def word2vec(self):
        from gensim import models
        w = models.KeyedVectors.load_word2vec_format(
            r'Word2Vec/GoogleNews-vectors-negative300.bin.gz', binary=True)
        return w.similarity(self.s1, self.s2)

    def ngram(self, n):
        if self.s1 == "null" or self.s2 == "null":
            return 0
        elif self.s1 == self.s2:
            return 1
        s1_list = ["".join(i) for i in nltkngram(self.s1, n)]
        s2_list = ["".join(i) for i in nltkngram(self.s2, n)]
        number_same = len(set(s1_list) & set(s2_list))
        below = min(len(self.s1), len(self.s2)) - n + 1
        return number_same / below


    def ngram2(self, n):
        if self.s1 == "null" or self.s2 == "null":
            return 0
        elif self.s1 == self.s2:
            return 1
        s1_list = ["".join(i) for i in nltkngram(self.s1, n)]
        s2_list = ["".join(i) for i in nltkngram(self.s2, n)]
        number_same = len(set(s1_list) & set(s2_list))
        below = len(self.s1) + len(self.s2)
        return 2 * number_same / below

def getIndex(onto_classes, id):
    DF = pd.DataFrame(onto_classes)[0]
    return (DF[DF == id].index)[0]

def similarity_flooding(onto1_classes, onto2_classes, onto1_dataproperties, onto2_dataproperties,
                        onto1_objectproperties, onto2_objectproperties):

    simi_classes = np.zeros((len(onto1_classes), len(onto2_classes)))
    simi_dataproperties = np.zeros((len(onto1_dataproperties), len(onto2_dataproperties)))
    simi_objectproperties = np.zeros((len(onto1_objectproperties), len(onto2_objectproperties)))

    # determine anchor
    for index_i, i in enumerate(onto1_classes):
        for index_j, j in enumerate(onto2_classes):
            if i[0] == j[0] or i[2] == j[2]:
                if i[2] != "null" and j[2] != "null":
                    simi_classes[index_i][index_j] = 1
    # propagation
    iteration = 3
    while iteration > 0:
        for index_i, i in enumerate(onto1_classes):
            for index_j, j in enumerate(onto2_classes):
                if simi_classes[index_i][index_j]:
                    son1 = i[4]
                    son2 = j[4]
                    if son1:
                        if " " in son1:
                            subclasses1 = son1.split(" ")
                        else:
                            subclasses1 = son1
                    else:
                        continue
                    if son2:
                        if " " in son2:
                            subclasses2 = son2.split(" ")
                        else:
                            subclasses2 = son2
                    else:
                        continue

                    for sub1 in subclasses1:
                        for sub2 in subclasses2:
                            sub1_Index = getIndex(onto1_classes, sub1)
                            sub2_Index = getIndex(onto2_classes, sub2)
                            if simi_classes[sub1_Index][sub2_Index]:
                                continue
                            else:
                                simi_classes[sub1_Index][sub2_Index] = simi_classes[index_i][index_j] / (
                                    len(list(set(subclasses1 + subclasses2))))
        iteration -= 1

    for index_i, i in enumerate(onto1_dataproperties):
        for index_j, j in enumerate(onto2_dataproperties):
            if i[0] == j[0] or i[2] == j[2]:
                simi_dataproperties[index_i][index_j] = 1

    domains1 = {}
    for index_i, i in enumerate(onto1_dataproperties):
        if not domains1.get(i[3]):
            domains1.update({i[3]: 1})
        else:
            domains1[i[3]] += 1
    domains2 = {}
    for index_i, i in enumerate(onto2_dataproperties):
        if not domains2.get(i[3]):
            domains2.update({i[3]: 1})
        else:
            domains2[i[3]] += 1

    for index_i, i in enumerate(onto1_dataproperties):
        for index_j, j in enumerate(onto2_dataproperties):
            if not simi_dataproperties[index_i][index_j]:
                num1 = domains1[i[3]]
                num2 = domains2[j[3]]
                try:
                    sub1_Index = getIndex(onto1_classes, i[3])
                    sub2_Index = getIndex(onto2_classes, j[3])
                except:
                    continue

                if simi_classes[sub1_Index][sub2_Index]:
                    simi_dataproperties[index_i][index_j] = simi_classes[sub1_Index][sub2_Index] / min(num1, num2)

    for index_i, i in enumerate(onto1_objectproperties):
        for index_j, j in enumerate(onto2_objectproperties):
            if i[0] == j[0] or i[2] == j[2]:
                simi_objectproperties[index_i][index_j] = 1

    domains1 = {}
    range1 = {}
    for index_i, i in enumerate(onto1_objectproperties):
        if not domains1.get(i[3]):
            domains1.update({i[3]: 1})
        else:
            domains1[i[3]] += 1
        if not range1.get(i[4]):
            range1.update({i[4]: 1})
        else:
            range1[i[4]] += 1
    domains2 = {}
    range2 = {}
    for index_i, i in enumerate(onto2_objectproperties):
        if not domains2.get(i[3]):
            domains2.update({i[3]: 1})
        else:
            domains2[i[3]] += 1
        if not range2.get(i[4]):
            range2.update({i[4]: 1})
        else:
            range2[i[4]] += 1

    for index_i, i in enumerate(onto1_objectproperties):
        for index_j, j in enumerate(onto2_objectproperties):
            if not simi_objectproperties[index_i][index_j]:
                try:
                    sub1_Index_domain = getIndex(onto1_classes, i[3])
                    sub2_Index_domain = getIndex(onto2_classes, j[3])
                except:
                    continue
                try:
                    sub1_Index_range = getIndex(onto1_classes, i[4])
                    sub2_Index_range = getIndex(onto2_classes, j[4])
                except:
                    continue

                if simi_classes[sub1_Index_domain][sub2_Index_domain]:
                    simi_domain = simi_classes[sub1_Index_domain][sub2_Index_domain] / min(domains1[i[3]],
                                                                                           domains2[j[3]])
                else:
                    simi_domain = 0
                if simi_classes[sub1_Index_range][sub2_Index_range]:
                    simi_range = simi_classes[sub1_Index_range][sub2_Index_range] / min(range1[i[4]], range2[j[4]])
                else:
                    simi_range = 0
                simi_objectproperties[index_i][index_j] = simi_domain + simi_range

    return simi_classes, simi_dataproperties, simi_objectproperties


def word_tokenize(word):
    print(wordninja.split(word))


def word_stemmer(word):
    import nltk
    from nltk.stem.porter import PorterStemmer
    porter_stemmer = PorterStemmer()
    print("Actual: %s  Stem: %s" % (word, porter_stemmer.stem(word)))


def word_lemmatize(word):
    from nltk.stem import WordNetLemmatizer
    wordnet_lemmatizer = WordNetLemmatizer()
    print("Actual: %s  Lemma: %s" % (word, wordnet_lemmatizer.lemmatize(word)))


if __name__ == "__main__":
    simi = similirites("name", "names")
    print("similarities", simi.Levensh(), simi.Jaro(), simi.wordnet(), simi.ngram(2), simi.ngram2(2))

